"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var monthsCache = new Map();
function setDay(date, day) {
    var newDate = new Date(date.getTime());
    newDate.setDate(day);
    return newDate;
}
exports.setDay = setDay;
function addDays(date, days) {
    return setDay(date, date.getDate() + days);
}
exports.addDays = addDays;
function setMonth(date, month) {
    var newDate = new Date(date.getTime());
    newDate.setMonth(month);
    return newDate;
}
exports.setMonth = setMonth;
function addMonths(date, months) {
    return setMonth(date, date.getMonth() + months);
}
exports.addMonths = addMonths;
function setYear(date, year) {
    var newDate = new Date(date.getTime());
    newDate.setFullYear(year);
    return newDate;
}
exports.setYear = setYear;
function startOfMonth(date) {
    var newDate = new Date(date.getTime());
    newDate.setDate(1);
    newDate.setHours(0, 0, 0, 0);
    return newDate;
}
exports.startOfMonth = startOfMonth;
function endOfMonth(date) {
    var newDate = new Date(date.getTime());
    newDate.setFullYear(date.getFullYear(), date.getMonth() + 1, 0);
    newDate.setHours(23, 59, 59, 999);
    return newDate;
}
exports.endOfMonth = endOfMonth;
function isSameMonth(dateLeft, dateRight) {
    return (dateLeft.getFullYear() === dateRight.getFullYear() &&
        dateLeft.getMonth() === dateRight.getMonth());
}
exports.isSameMonth = isSameMonth;
function isSameDay(dateLeft, dateRight) {
    return (dateLeft.getFullYear() === dateRight.getFullYear() &&
        dateLeft.getMonth() === dateRight.getMonth() &&
        dateLeft.getDate() === dateRight.getDate());
}
exports.isSameDay = isSameDay;
function isDayAfter(date, dateToCompare) {
    var newDate = new Date(date.getTime());
    var newDateToCompare = new Date(dateToCompare.getTime());
    newDate.setHours(0, 0, 0, 0);
    newDateToCompare.setHours(0, 0, 0, 0);
    return newDate.getTime() > newDateToCompare.getTime();
}
exports.isDayAfter = isDayAfter;
function isDayBefore(date, dateToCompare) {
    var newDate = new Date(date.getTime());
    var newDateToCompare = new Date(dateToCompare.getTime());
    newDate.setHours(0, 0, 0, 0);
    newDateToCompare.setHours(0, 0, 0, 0);
    return newDate.getTime() < newDateToCompare.getTime();
}
exports.isDayBefore = isDayBefore;
function callIfExists(callback) {
    var args = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        args[_i - 1] = arguments[_i];
    }
    if (typeof callback === 'function') {
        callback.apply(void 0, args);
    }
}
exports.callIfExists = callIfExists;
function getDatesofMonth(month) {
    var firstDate = startOfMonth(month);
    var key = firstDate.getFullYear() + "-" + firstDate.getMonth();
    var data = monthsCache.get(key);
    if (data) {
        return data;
    }
    /*
        Logic to see how many weeks we will have to render, this can be easily done by :
        Math.ceil of ((Get the first day in current month + total number of days in month) / (No. of days in week i.e. 7))
    */
    var firstDayInCurrentMonth = firstDate.getDay();
    var lastDate = endOfMonth(month);
    var noOfWeeks = Math.ceil((firstDayInCurrentMonth + lastDate.getDate()) / 7);
    var newdata = Array.from({ length: noOfWeeks }, function (_1, i) {
        var week = Array.from({ length: 7 }, function (_2, j) {
            var day = addDays(firstDate, i * 7 + j - firstDayInCurrentMonth);
            return { date: day, inCurrentMonth: isSameMonth(firstDate, day) };
        });
        return week;
    });
    monthsCache.set(key, newdata);
    return newdata;
}
exports.getDatesofMonth = getDatesofMonth;
